/* Sliver - a BPEL execution engine for lightweight and mobile devices.
 * Copyright (C) 2006, Washington University in Saint Louis
 * By Gregory Hackmann.
 *
 * Washington University states that Sliver is free software;
 * you can redistribute it and/or modify it under the terms of
 * the current version of the GNU Lesser General Public License
 * as published by the Free Software Foundation.
 *
 * Sliver is distributed in the hope that it will be useful, but
 * THERE ARE NO WARRANTIES, WHETHER ORAL OR WRITTEN, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.
 *
 * YOU UNDERSTAND THAT SLIVER IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. THERE ARE NO
 * WARRANTIES AND NO REPRESENTATION THAT SLIVER IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER
 * PROPRIETARY RIGHTS.  THERE ARE NO WARRANTIES THAT SOFTWARE IS
 * FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS",
 * OR OTHER HARMFUL CODE.
 *
 * YOU ASSUME THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF
 * INFORMATION GENERATED USING SOFTWARE. By using Sliver you agree to
 * indemnify, defend, and hold harmless WU, its employees, officers and
 * agents from any and all claims, costs, or liabilities, including
 * attorneys fees and court costs at both the trial and appellate levels
 * for any loss, damage, or injury caused by your actions or actions of
 * your officers, servants, agents or third parties acting on behalf or
 * under authorization from you, as a result of using Sliver.
 *
 * See the GNU Lesser General Public License for more details, which can
 * be found here: http://www.gnu.org/copyleft/lesser.html
 */

package edu.wustl.mobilab.sliver.bpel;

import org.ksoap2.*;
import org.kxml2.kdom.*;

import edu.wustl.mobilab.sliver.soap.*;

/**
 * Encapsulates uncaught faults thrown during the BPEL process execution.
 * 
 * @author Greg Hackmann
 */
public class BPELServerFault extends Exception
{
	/**
	 * The uncaught fault.
	 */
	transient private final Fault fault;
	
	/**
	 * Creates a new BPELServerFault.
	 * 
	 * @param message the fault message
	 */
	private BPELServerFault(String message)
	{
		fault = new Fault("Server", message);
	}
	
	/**
	 * Creates a new BPELServerFault.
	 * 
	 * @param message the fault message
	 * @param cause the uncaught {@link SoapFault} that was thrown
	 */
	BPELServerFault(String message, SoapFault cause)
	{
		this(message);
		fault.addFaultActor(cause.faultactor);
		// Copy the fault actor
		
		Object detail = cause.detail.getChild(0);
		if(detail instanceof Element)
		{
			Element element = (Element)detail;
			fault.addDetail(element.getChild(0).toString());
		}
		else
			fault.addDetail(detail.toString());
		// Copy the fault detail
	}
	
	/**
	 * Creates a new BPELServerFault.
	 * 
	 * @param cause the uncaught exception that was thrown
	 */
	BPELServerFault(Throwable cause)
	{
		this(cause.getMessage());
		if(cause instanceof SerializableException)
			fault.addDetail((SerializableException)cause);
		else
			fault.addDetail(cause.toString());
		// Add the throwable as a fault detail
	}
	
	/**
	 * Gets the uncaught fault that caused this Exception.
	 * 
	 * @return the uncaught fault that caused this Exception
	 */
	Fault getFault()
	{
		return fault;
	}
}
